@@ -1,6 +1,8 @@
 from jupyter_server.base.handlers import JupyterHandler
-from jupyter_server.extension.handler import ExtensionHandlerJinjaMixin
-from jupyter_server.extension.handler import ExtensionHandlerMixin
+from jupyter_server.extension.handler import (
+    ExtensionHandlerJinjaMixin,
+    ExtensionHandlerMixin,
+)
 from jupyter_server.utils import url_escape
 
 
@@ -9,9 +11,9 @@ class ParameterHandler(ExtensionHandlerMixin, JupyterHandler):
         var1 = self.get_argument("var1", default=None)
         components = [x for x in self.request.path.split("/") if x]
         self.write("<h1>Hello Simple App 2 from Handler.</h1>")
-        self.write("<p>matched_part: {}</p>".format(url_escape(matched_part)))
-        self.write("<p>var1: {}</p>".format(url_escape(var1)))
-        self.write("<p>components: {}</p>".format(components))
+        self.write(f"<p>matched_part: {url_escape(matched_part)}</p>")
+        self.write(f"<p>var1: {url_escape(var1)}</p>")
+        self.write(f"<p>components: {components}</p>")
 
 
 class BaseTemplateHandler(ExtensionHandlerJinjaMixin, ExtensionHandlerMixin, JupyterHandler):
@@ -31,4 +33,4 @@ class TemplateHandler(BaseTemplateHandler):
 
 class ErrorHandler(BaseTemplateHandler):
     def get(self, path):
-        self.write(self.render_template("error.html"))
+        self.write_error(400)
