@@ -1,4 +1,3 @@
-# encoding: utf-8
 """
 Timezone utilities
 
@@ -6,9 +5,7 @@ Just UTC-awareness right now
 """
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-from datetime import datetime
-from datetime import timedelta
-from datetime import tzinfo
+from datetime import datetime, timedelta, tzinfo
 
 # constant for zero offset
 ZERO = timedelta(0)
