@@ -9,14 +9,16 @@ from jupyter_core.utils import ensure_dir_exists
 from tornado.web import HTTPError
 from traitlets import Unicode
 
-from .checkpoints import AsyncCheckpoints
-from .checkpoints import AsyncGenericCheckpointsMixin
-from .checkpoints import Checkpoints
-from .checkpoints import GenericCheckpointsMixin
-from .fileio import AsyncFileManagerMixin
-from .fileio import FileManagerMixin
 from jupyter_server import _tz as tz
 
+from .checkpoints import (
+    AsyncCheckpoints,
+    AsyncGenericCheckpointsMixin,
+    Checkpoints,
+    GenericCheckpointsMixin,
+)
+from .fileio import AsyncFileManagerMixin, FileManagerMixin
+
 
 class FileCheckpoints(FileManagerMixin, Checkpoints):
     """
@@ -49,7 +51,7 @@ class FileCheckpoints(FileManagerMixin, Checkpoints):
     # ContentsManager-dependent checkpoint API
     def create_checkpoint(self, contents_mgr, path):
         """Create a checkpoint."""
-        checkpoint_id = u"checkpoint"
+        checkpoint_id = "checkpoint"
         src_path = contents_mgr._get_os_path(path)
         dest_path = self.checkpoint_path(checkpoint_id, path)
         self._copy(src_path, dest_path)
@@ -106,7 +108,7 @@ class FileCheckpoints(FileManagerMixin, Checkpoints):
         parent, name = ("/" + path).rsplit("/", 1)
         parent = parent.strip("/")
         basename, ext = os.path.splitext(name)
-        filename = u"{name}-{checkpoint_id}{ext}".format(
+        filename = "{name}-{checkpoint_id}{ext}".format(
             name=basename,
             checkpoint_id=checkpoint_id,
             ext=ext,
@@ -130,13 +132,13 @@ class FileCheckpoints(FileManagerMixin, Checkpoints):
 
     # Error Handling
     def no_such_checkpoint(self, path, checkpoint_id):
-        raise HTTPError(404, u"Checkpoint does not exist: %s@%s" % (path, checkpoint_id))
+        raise HTTPError(404, f"Checkpoint does not exist: {path}@{checkpoint_id}")
 
 
 class AsyncFileCheckpoints(FileCheckpoints, AsyncFileManagerMixin, AsyncCheckpoints):
     async def create_checkpoint(self, contents_mgr, path):
         """Create a checkpoint."""
-        checkpoint_id = u"checkpoint"
+        checkpoint_id = "checkpoint"
         src_path = contents_mgr._get_os_path(path)
         dest_path = self.checkpoint_path(checkpoint_id, path)
         await self._copy(src_path, dest_path)
@@ -207,7 +209,7 @@ class GenericFileCheckpoints(GenericCheckpointsMixin, FileCheckpoints):
         """Create a checkpoint from the current content of a file."""
         path = path.strip("/")
         # only the one checkpoint ID:
-        checkpoint_id = u"checkpoint"
+        checkpoint_id = "checkpoint"
         os_checkpoint_path = self.checkpoint_path(checkpoint_id, path)
         self.log.debug("creating checkpoint for %s", path)
         with self.perm_to_403():
@@ -220,7 +222,7 @@ class GenericFileCheckpoints(GenericCheckpointsMixin, FileCheckpoints):
         """Create a checkpoint from the current content of a notebook."""
         path = path.strip("/")
         # only the one checkpoint ID:
-        checkpoint_id = u"checkpoint"
+        checkpoint_id = "checkpoint"
         os_checkpoint_path = self.checkpoint_path(checkpoint_id, path)
         self.log.debug("creating checkpoint for %s", path)
         with self.perm_to_403():
@@ -273,7 +275,7 @@ class AsyncGenericFileCheckpoints(AsyncGenericCheckpointsMixin, AsyncFileCheckpo
         """Create a checkpoint from the current content of a file."""
         path = path.strip("/")
         # only the one checkpoint ID:
-        checkpoint_id = u"checkpoint"
+        checkpoint_id = "checkpoint"
         os_checkpoint_path = self.checkpoint_path(checkpoint_id, path)
         self.log.debug("creating checkpoint for %s", path)
         with self.perm_to_403():
@@ -286,7 +288,7 @@ class AsyncGenericFileCheckpoints(AsyncGenericCheckpointsMixin, AsyncFileCheckpo
         """Create a checkpoint from the current content of a notebook."""
         path = path.strip("/")
         # only the one checkpoint ID:
-        checkpoint_id = u"checkpoint"
+        checkpoint_id = "checkpoint"
         os_checkpoint_path = self.checkpoint_path(checkpoint_id, path)
         self.log.debug("creating checkpoint for %s", path)
         with self.perm_to_403():
