@@ -11,29 +11,19 @@ from datetime import datetime
 
 import nbformat
 from anyio.to_thread import run_sync
-from ipython_genutils.importstring import import_item
-from jupyter_core.paths import exists
-from jupyter_core.paths import is_file_hidden
-from jupyter_core.paths import is_hidden
+from jupyter_core.paths import exists, is_file_hidden, is_hidden
 from send2trash import send2trash
 from tornado import web
-from traitlets import Any
-from traitlets import Bool
-from traitlets import default
-from traitlets import TraitError
-from traitlets import Unicode
-from traitlets import validate
+from traitlets import Bool, TraitError, Unicode, default, validate
 
-from .filecheckpoints import AsyncFileCheckpoints
-from .filecheckpoints import FileCheckpoints
-from .fileio import AsyncFileManagerMixin
-from .fileio import FileManagerMixin
-from .manager import AsyncContentsManager
-from .manager import ContentsManager
 from jupyter_server import _tz as tz
 from jupyter_server.base.handlers import AuthenticatedFileHandler
 from jupyter_server.transutils import _i18n
 
+from .filecheckpoints import AsyncFileCheckpoints, FileCheckpoints
+from .fileio import AsyncFileManagerMixin, FileManagerMixin
+from .manager import AsyncContentsManager, ContentsManager
+
 try:
     from os.path import samefile
 except ImportError:
@@ -54,48 +44,6 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         except AttributeError:
             return os.getcwd()
 
-    post_save_hook = Any(
-        None,
-        config=True,
-        allow_none=True,
-        help="""Python callable or importstring thereof
-
-        to be called on the path of a file just saved.
-
-        This can be used to process the file on disk,
-        such as converting the notebook to a script or HTML via nbconvert.
-
-        It will be called as (all arguments passed by keyword)::
-
-            hook(os_path=os_path, model=model, contents_manager=instance)
-
-        - path: the filesystem path to the file just written
-        - model: the model representing the file
-        - contents_manager: this ContentsManager instance
-        """,
-    )
-
-    @validate("post_save_hook")
-    def _validate_post_save_hook(self, proposal):
-        value = proposal["value"]
-        if isinstance(value, str):
-            value = import_item(value)
-        if not callable(value):
-            raise TraitError("post_save_hook must be callable")
-        return value
-
-    def run_post_save_hook(self, model, os_path):
-        """Run the post-save hook if defined, and log errors"""
-        if self.post_save_hook:
-            try:
-                self.log.debug("Running post-save hook on %s", os_path)
-                self.post_save_hook(os_path=os_path, model=model, contents_manager=self)
-            except Exception as e:
-                self.log.error("Post-save hook failed o-n %s", os_path, exc_info=True)
-                raise web.HTTPError(
-                    500, u"Unexpected error while running post hook save: %s" % e
-                ) from e
-
     @validate("root_dir")
     def _validate_root_dir(self, proposal):
         """Do a bit of validation of the root_dir."""
@@ -155,6 +103,28 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path=path)
         return is_hidden(os_path, self.root_dir)
 
+    def is_writable(self, path):
+        """Does the API style path correspond to a writable directory or file?
+
+        Parameters
+        ----------
+        path : string
+            The path to check. This is an API path (`/` separated,
+            relative to root_dir).
+
+        Returns
+        -------
+        hidden : bool
+            Whether the path exists and is writable.
+        """
+        path = path.strip("/")
+        os_path = self._get_os_path(path=path)
+        try:
+            return os.access(os_path, os.W_OK)
+        except OSError:
+            self.log.error("Failed to check write permissions on %s", os_path)
+            return False
+
     def file_exists(self, path):
         """Returns True if the file exists, else returns False.
 
@@ -218,6 +188,12 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path)
         info = os.lstat(os_path)
 
+        four_o_four = "file or directory does not exist: %r" % path
+
+        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+            self.log.info("Refusing to serve hidden file or directory %r, via 404 Error", os_path)
+            raise web.HTTPError(404, four_o_four)
+
         try:
             # size of file
             size = info.st_size
@@ -251,12 +227,8 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         model["format"] = None
         model["mimetype"] = None
         model["size"] = size
+        model["writable"] = self.is_writable(path)
 
-        try:
-            model["writable"] = os.access(os_path, os.W_OK)
-        except OSError:
-            self.log.error("Failed to check write permissions on %s", os_path)
-            model["writable"] = False
         return model
 
     def _dir_model(self, path, content=True):
@@ -266,7 +238,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """
         os_path = self._get_os_path(path)
 
-        four_o_four = u"directory does not exist: %r" % path
+        four_o_four = "directory does not exist: %r" % path
 
         if not os.path.isdir(os_path):
             raise web.HTTPError(404, four_o_four)
@@ -308,7 +280,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
                 try:
                     if self.should_list(name):
                         if self.allow_hidden or not is_file_hidden(os_path, stat_res=st):
-                            contents.append(self.get(path="%s/%s" % (path, name), content=False))
+                            contents.append(self.get(path=f"{path}/{name}", content=False))
                 except OSError as e:
                     # ELOOP: recursive symlink, also don't show failure due to permissions
                     if e.errno not in [errno.ELOOP, errno.EACCES]:
@@ -365,11 +337,14 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path)
 
         if content:
-            nb = self._read_notebook(os_path, as_version=4)
+            validation_error: dict = {}
+            nb = self._read_notebook(
+                os_path, as_version=4, capture_validation_error=validation_error
+            )
             self.mark_trusted_cells(nb, path)
             model["content"] = nb
             model["format"] = "json"
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error)
 
         return model
 
@@ -396,16 +371,21 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             of the file or directory as well.
         """
         path = path.strip("/")
+        os_path = self._get_os_path(path)
+        four_o_four = "file or directory does not exist: %r" % path
 
         if not self.exists(path):
-            raise web.HTTPError(404, u"No such file or directory: %s" % path)
+            raise web.HTTPError(404, four_o_four)
+
+        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+            self.log.info("Refusing to serve hidden file or directory %r, via 404 Error", os_path)
+            raise web.HTTPError(404, four_o_four)
 
-        os_path = self._get_os_path(path)
         if os.path.isdir(os_path):
             if type not in (None, "directory"):
                 raise web.HTTPError(
                     400,
-                    u"%s is a directory, not a %s" % (path, type),
+                    f"{path} is a directory, not a {type}",
                     reason="bad type",
                 )
             model = self._dir_model(path, content=content)
@@ -413,19 +393,19 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             model = self._notebook_model(path, content=content)
         else:
             if type == "directory":
-                raise web.HTTPError(400, u"%s is not a directory" % path, reason="bad type")
+                raise web.HTTPError(400, "%s is not a directory" % path, reason="bad type")
             model = self._file_model(path, content=content, format=format)
         return model
 
     def _save_directory(self, os_path, model, path=""):
         """create a directory"""
         if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
-            raise web.HTTPError(400, u"Cannot create hidden directory %r" % os_path)
+            raise web.HTTPError(400, "Cannot create directory %r" % os_path)
         if not os.path.exists(os_path):
             with self.perm_to_403():
                 os.mkdir(os_path)
         elif not os.path.isdir(os_path):
-            raise web.HTTPError(400, u"Not a directory: %s" % (os_path))
+            raise web.HTTPError(400, "Not a directory: %s" % (os_path))
         else:
             self.log.debug("Directory %r already exists", os_path)
 
@@ -433,21 +413,26 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """Save the file model and return the model with no content."""
         path = path.strip("/")
 
+        self.run_pre_save_hooks(model=model, path=path)
+
         if "type" not in model:
-            raise web.HTTPError(400, u"No file type provided")
+            raise web.HTTPError(400, "No file type provided")
         if "content" not in model and model["type"] != "directory":
-            raise web.HTTPError(400, u"No file content provided")
+            raise web.HTTPError(400, "No file content provided")
 
         os_path = self._get_os_path(path)
+
+        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+            raise web.HTTPError(400, f"Cannot create file or directory {os_path!r}")
+
         self.log.debug("Saving %s", os_path)
 
-        self.run_pre_save_hook(model=model, path=path)
-
+        validation_error: dict = {}
         try:
             if model["type"] == "notebook":
                 nb = nbformat.from_dict(model["content"])
                 self.check_and_sign(nb, path)
-                self._save_notebook(os_path, nb)
+                self._save_notebook(os_path, nb, capture_validation_error=validation_error)
                 # One checkpoint should always exist for notebooks.
                 if not self.checkpoints.list_checkpoints(path):
                     self.create_checkpoint(path)
@@ -461,21 +446,19 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         except web.HTTPError:
             raise
         except Exception as e:
-            self.log.error(u"Error while saving file: %s %s", path, e, exc_info=True)
-            raise web.HTTPError(
-                500, u"Unexpected error while saving file: %s %s" % (path, e)
-            ) from e
+            self.log.error("Error while saving file: %s %s", path, e, exc_info=True)
+            raise web.HTTPError(500, f"Unexpected error while saving file: {path} {e}") from e
 
         validation_message = None
         if model["type"] == "notebook":
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error=validation_error)
             validation_message = model.get("message", None)
 
         model = self.get(path, content=False)
         if validation_message:
             model["message"] = validation_message
 
-        self.run_post_save_hook(model=model, os_path=os_path)
+        self.run_post_save_hooks(model=model, os_path=os_path)
 
         return model
 
@@ -484,8 +467,13 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         path = path.strip("/")
         os_path = self._get_os_path(path)
         rm = os.unlink
-        if not os.path.exists(os_path):
-            raise web.HTTPError(404, u"File or directory does not exist: %s" % os_path)
+
+        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+            raise web.HTTPError(400, f"Cannot delete file or directory {os_path!r}")
+
+        four_o_four = "file or directory does not exist: %r" % path
+        if not self.exists(path):
+            raise web.HTTPError(404, four_o_four)
 
         def _check_trash(os_path):
             if sys.platform in {"win32", "darwin"}:
@@ -512,24 +500,26 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             if not self.always_delete_dir and sys.platform == "win32" and is_non_empty_dir(os_path):
                 # send2trash can really delete files on Windows, so disallow
                 # deleting non-empty files. See Github issue 3631.
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             if _check_trash(os_path):
-                self.log.debug("Sending %s to trash", os_path)
                 # Looking at the code in send2trash, I don't think the errors it
                 # raises let us distinguish permission errors from other errors in
-                # code. So for now, just let them all get logged as server errors.
+                # code. So for now, the "look before you leap" approach is used.
+                if not self.is_writable(path):
+                    raise web.HTTPError(403, "Permission denied: %s" % path)
+                self.log.debug("Sending %s to trash", os_path)
                 send2trash(os_path)
                 return
             else:
                 self.log.warning(
-                    "Skipping trash for %s, on different device " "to home directory",
+                    "Skipping trash for %s, on different device to home directory",
                     os_path,
                 )
 
         if os.path.isdir(os_path):
             # Don't permanently delete non-empty directories.
             if not self.always_delete_dir and is_non_empty_dir(os_path):
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             self.log.debug("Removing directory %s", os_path)
             with self.perm_to_403():
                 shutil.rmtree(os_path)
@@ -548,9 +538,14 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         new_os_path = self._get_os_path(new_path)
         old_os_path = self._get_os_path(old_path)
 
+        if (
+            is_hidden(old_os_path, self.root_dir) or is_hidden(new_os_path, self.root_dir)
+        ) and not self.allow_hidden:
+            raise web.HTTPError(400, f"Cannot rename file or directory {old_os_path!r}")
+
         # Should we proceed with the move?
         if os.path.exists(new_os_path) and not samefile(old_os_path, new_os_path):
-            raise web.HTTPError(409, u"File already exists: %s" % new_path)
+            raise web.HTTPError(409, "File already exists: %s" % new_path)
 
         # Move the file
         try:
@@ -559,7 +554,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         except web.HTTPError:
             raise
         except Exception as e:
-            raise web.HTTPError(500, u"Unknown error renaming file: %s %s" % (old_path, e)) from e
+            raise web.HTTPError(500, f"Unknown error renaming file: {old_path} {e}") from e
 
     def info_string(self):
         return _i18n("Serving notebooks from local directory: %s") % self.root_dir
@@ -587,7 +582,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         """
         os_path = self._get_os_path(path)
 
-        four_o_four = u"directory does not exist: %r" % path
+        four_o_four = "directory does not exist: %r" % path
 
         if not os.path.isdir(os_path):
             raise web.HTTPError(404, four_o_four)
@@ -630,9 +625,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
                 try:
                     if self.should_list(name):
                         if self.allow_hidden or not is_file_hidden(os_path, stat_res=st):
-                            contents.append(
-                                await self.get(path="%s/%s" % (path, name), content=False)
-                            )
+                            contents.append(await self.get(path=f"{path}/{name}", content=False))
                 except OSError as e:
                     # ELOOP: recursive symlink, also don't show failure due to permissions
                     if e.errno not in [errno.ELOOP, errno.EACCES]:
@@ -689,11 +682,14 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         os_path = self._get_os_path(path)
 
         if content:
-            nb = await self._read_notebook(os_path, as_version=4)
+            validation_error: dict = {}
+            nb = await self._read_notebook(
+                os_path, as_version=4, capture_validation_error=validation_error
+            )
             self.mark_trusted_cells(nb, path)
             model["content"] = nb
             model["format"] = "json"
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error)
 
         return model
 
@@ -722,14 +718,14 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         path = path.strip("/")
 
         if not self.exists(path):
-            raise web.HTTPError(404, u"No such file or directory: %s" % path)
+            raise web.HTTPError(404, "No such file or directory: %s" % path)
 
         os_path = self._get_os_path(path)
         if os.path.isdir(os_path):
             if type not in (None, "directory"):
                 raise web.HTTPError(
                     400,
-                    u"%s is a directory, not a %s" % (path, type),
+                    f"{path} is a directory, not a {type}",
                     reason="bad type",
                 )
             model = await self._dir_model(path, content=content)
@@ -737,19 +733,19 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
             model = await self._notebook_model(path, content=content)
         else:
             if type == "directory":
-                raise web.HTTPError(400, u"%s is not a directory" % path, reason="bad type")
+                raise web.HTTPError(400, "%s is not a directory" % path, reason="bad type")
             model = await self._file_model(path, content=content, format=format)
         return model
 
     async def _save_directory(self, os_path, model, path=""):
         """create a directory"""
         if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
-            raise web.HTTPError(400, u"Cannot create hidden directory %r" % os_path)
+            raise web.HTTPError(400, "Cannot create hidden directory %r" % os_path)
         if not os.path.exists(os_path):
             with self.perm_to_403():
                 await run_sync(os.mkdir, os_path)
         elif not os.path.isdir(os_path):
-            raise web.HTTPError(400, u"Not a directory: %s" % (os_path))
+            raise web.HTTPError(400, "Not a directory: %s" % (os_path))
         else:
             self.log.debug("Directory %r already exists", os_path)
 
@@ -757,21 +753,22 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         """Save the file model and return the model with no content."""
         path = path.strip("/")
 
+        self.run_pre_save_hook(model=model, path=path)
+
         if "type" not in model:
-            raise web.HTTPError(400, u"No file type provided")
+            raise web.HTTPError(400, "No file type provided")
         if "content" not in model and model["type"] != "directory":
-            raise web.HTTPError(400, u"No file content provided")
+            raise web.HTTPError(400, "No file content provided")
 
         os_path = self._get_os_path(path)
         self.log.debug("Saving %s", os_path)
 
-        self.run_pre_save_hook(model=model, path=path)
-
+        validation_error: dict = {}
         try:
             if model["type"] == "notebook":
                 nb = nbformat.from_dict(model["content"])
                 self.check_and_sign(nb, path)
-                await self._save_notebook(os_path, nb)
+                await self._save_notebook(os_path, nb, capture_validation_error=validation_error)
                 # One checkpoint should always exist for notebooks.
                 if not (await self.checkpoints.list_checkpoints(path)):
                     await self.create_checkpoint(path)
@@ -785,21 +782,19 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         except web.HTTPError:
             raise
         except Exception as e:
-            self.log.error(u"Error while saving file: %s %s", path, e, exc_info=True)
-            raise web.HTTPError(
-                500, u"Unexpected error while saving file: %s %s" % (path, e)
-            ) from e
+            self.log.error("Error while saving file: %s %s", path, e, exc_info=True)
+            raise web.HTTPError(500, f"Unexpected error while saving file: {path} {e}") from e
 
         validation_message = None
         if model["type"] == "notebook":
-            self.validate_notebook_model(model)
+            self.validate_notebook_model(model, validation_error=validation_error)
             validation_message = model.get("message", None)
 
         model = await self.get(path, content=False)
         if validation_message:
             model["message"] = validation_message
 
-        self.run_post_save_hook(model=model, os_path=os_path)
+        self.run_post_save_hooks(model=model, os_path=os_path)
 
         return model
 
@@ -808,8 +803,12 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         path = path.strip("/")
         os_path = self._get_os_path(path)
         rm = os.unlink
+
+        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+            raise web.HTTPError(400, f"Cannot delete file or directory {os_path!r}")
+
         if not os.path.exists(os_path):
-            raise web.HTTPError(404, u"File or directory does not exist: %s" % os_path)
+            raise web.HTTPError(404, "File or directory does not exist: %s" % os_path)
 
         async def _check_trash(os_path):
             if sys.platform in {"win32", "darwin"}:
@@ -841,24 +840,26 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
             ):
                 # send2trash can really delete files on Windows, so disallow
                 # deleting non-empty files. See Github issue 3631.
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             if await _check_trash(os_path):
-                self.log.debug("Sending %s to trash", os_path)
                 # Looking at the code in send2trash, I don't think the errors it
                 # raises let us distinguish permission errors from other errors in
-                # code. So for now, just let them all get logged as server errors.
+                # code. So for now, the "look before you leap" approach is used.
+                if not self.is_writable(path):
+                    raise web.HTTPError(403, "Permission denied: %s" % path)
+                self.log.debug("Sending %s to trash", os_path)
                 send2trash(os_path)
                 return
             else:
                 self.log.warning(
-                    "Skipping trash for %s, on different device " "to home directory",
+                    "Skipping trash for %s, on different device to home directory",
                     os_path,
                 )
 
         if os.path.isdir(os_path):
             # Don't permanently delete non-empty directories.
             if not self.always_delete_dir and await is_non_empty_dir(os_path):
-                raise web.HTTPError(400, u"Directory %s not empty" % os_path)
+                raise web.HTTPError(400, "Directory %s not empty" % os_path)
             self.log.debug("Removing directory %s", os_path)
             with self.perm_to_403():
                 await run_sync(shutil.rmtree, os_path)
@@ -877,9 +878,14 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         new_os_path = self._get_os_path(new_path)
         old_os_path = self._get_os_path(old_path)
 
+        if (
+            is_hidden(old_os_path, self.root_dir) or is_hidden(new_os_path, self.root_dir)
+        ) and not self.allow_hidden:
+            raise web.HTTPError(400, f"Cannot rename file or directory {old_os_path!r}")
+
         # Should we proceed with the move?
         if os.path.exists(new_os_path) and not samefile(old_os_path, new_os_path):
-            raise web.HTTPError(409, u"File already exists: %s" % new_path)
+            raise web.HTTPError(409, "File already exists: %s" % new_path)
 
         # Move the file
         try:
@@ -888,4 +894,22 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         except web.HTTPError:
             raise
         except Exception as e:
-            raise web.HTTPError(500, u"Unknown error renaming file: %s %s" % (old_path, e)) from e
+            raise web.HTTPError(500, f"Unknown error renaming file: {old_path} {e}") from e
+
+    async def dir_exists(self, path):
+        """Does a directory exist at the given path"""
+        path = path.strip("/")
+        os_path = self._get_os_path(path=path)
+        return os.path.isdir(os_path)
+
+    async def file_exists(self, path):
+        """Does a file exist at the given path"""
+        path = path.strip("/")
+        os_path = self._get_os_path(path)
+        return os.path.isfile(os_path)
+
+    async def is_hidden(self, path):
+        """Is path a hidden directory or file"""
+        path = path.strip("/")
+        os_path = self._get_os_path(path=path)
+        return is_hidden(os_path, self.root_dir)
