@@ -10,14 +10,14 @@ try:
     from jupyter_client.jsonutil import json_default
 except ImportError:
     from jupyter_client.jsonutil import date_default as json_default
+
 from tornado import web
 
-from jupyter_server.base.handlers import APIHandler
-from jupyter_server.base.handlers import JupyterHandler
-from jupyter_server.base.handlers import path_regex
-from jupyter_server.utils import ensure_async
-from jupyter_server.utils import url_escape
-from jupyter_server.utils import url_path_join
+from jupyter_server.auth import authorized
+from jupyter_server.base.handlers import APIHandler, JupyterHandler, path_regex
+from jupyter_server.utils import ensure_async, url_escape, url_path_join
+
+AUTH_RESOURCE = "contents"
 
 
 def validate_model(model, expect_content):
@@ -42,7 +42,7 @@ def validate_model(model, expect_content):
     if missing:
         raise web.HTTPError(
             500,
-            u"Missing Model Keys: {missing}".format(missing=missing),
+            f"Missing Model Keys: {missing}",
         )
 
     maybe_none_keys = ["content", "format"]
@@ -51,18 +51,22 @@ def validate_model(model, expect_content):
         if errors:
             raise web.HTTPError(
                 500,
-                u"Keys unexpectedly None: {keys}".format(keys=errors),
+                f"Keys unexpectedly None: {errors}",
             )
     else:
-        errors = {key: model[key] for key in maybe_none_keys if model[key] is not None}
+        errors = {key: model[key] for key in maybe_none_keys if model[key] is not None}  # type: ignore[assignment]
         if errors:
             raise web.HTTPError(
                 500,
-                u"Keys unexpectedly not None: {keys}".format(keys=errors),
+                f"Keys unexpectedly not None: {errors}",
             )
 
 
-class ContentsHandler(APIHandler):
+class ContentsAPIHandler(APIHandler):
+    auth_resource = AUTH_RESOURCE
+
+
+class ContentsHandler(ContentsAPIHandler):
     def location_url(self, path):
         """Return the full URL location of a file.
 
@@ -83,6 +87,7 @@ class ContentsHandler(APIHandler):
         self.finish(json.dumps(model, default=json_default))
 
     @web.authenticated
+    @authorized
     async def get(self, path=""):
         """Return a model for a file or directory.
 
@@ -90,17 +95,22 @@ class ContentsHandler(APIHandler):
         of the files and directories it contains.
         """
         path = path or ""
+        cm = self.contents_manager
+
         type = self.get_query_argument("type", default=None)
         if type not in {None, "directory", "file", "notebook"}:
-            raise web.HTTPError(400, u"Type %r is invalid" % type)
+            raise web.HTTPError(400, "Type %r is invalid" % type)
 
         format = self.get_query_argument("format", default=None)
         if format not in {None, "text", "base64"}:
-            raise web.HTTPError(400, u"Format %r is invalid" % format)
-        content = self.get_query_argument("content", default="1")
-        if content not in {"0", "1"}:
-            raise web.HTTPError(400, u"Content %r is invalid" % content)
-        content = int(content)
+            raise web.HTTPError(400, "Format %r is invalid" % format)
+        content_str = self.get_query_argument("content", default="1")
+        if content_str not in {"0", "1"}:
+            raise web.HTTPError(400, "Content %r is invalid" % content_str)
+        content = int(content_str or "")
+
+        if await ensure_async(cm.is_hidden(path)) and not cm.allow_hidden:
+            raise web.HTTPError(404, f"file or directory {path!r} does not exist")
 
         model = await ensure_async(
             self.contents_manager.get(
@@ -114,12 +124,24 @@ class ContentsHandler(APIHandler):
         self._finish_model(model, location=False)
 
     @web.authenticated
+    @authorized
     async def patch(self, path=""):
         """PATCH renames a file or directory without re-uploading content."""
         cm = self.contents_manager
         model = self.get_json_body()
         if model is None:
-            raise web.HTTPError(400, u"JSON body missing")
+            raise web.HTTPError(400, "JSON body missing")
+
+        old_path = model.get("path")
+        if (
+            old_path
+            and (
+                await ensure_async(cm.is_hidden(path)) or await ensure_async(cm.is_hidden(old_path))
+            )
+            and not cm.allow_hidden
+        ):
+            raise web.HTTPError(400, f"Cannot rename file or directory {path!r}")
+
         model = await ensure_async(cm.update(model, path))
         validate_model(model, expect_content=False)
         self._finish_model(model)
@@ -127,7 +149,7 @@ class ContentsHandler(APIHandler):
     async def _copy(self, copy_from, copy_to=None):
         """Copy a file, optionally specifying a target directory."""
         self.log.info(
-            u"Copying {copy_from} to {copy_to}".format(
+            "Copying {copy_from} to {copy_to}".format(
                 copy_from=copy_from,
                 copy_to=copy_to or "",
             )
@@ -139,7 +161,7 @@ class ContentsHandler(APIHandler):
 
     async def _upload(self, model, path):
         """Handle upload of a new file to path"""
-        self.log.info(u"Uploading file to %s", path)
+        self.log.info("Uploading file to %s", path)
         model = await ensure_async(self.contents_manager.new(model, path))
         self.set_status(201)
         validate_model(model, expect_content=False)
@@ -147,7 +169,7 @@ class ContentsHandler(APIHandler):
 
     async def _new_untitled(self, path, type="", ext=""):
         """Create a new, empty untitled entity"""
-        self.log.info(u"Creating new %s in %s", type or "file", path)
+        self.log.info("Creating new %s in %s", type or "file", path)
         model = await ensure_async(
             self.contents_manager.new_untitled(path=path, type=type, ext=ext)
         )
@@ -159,12 +181,13 @@ class ContentsHandler(APIHandler):
         """Save an existing file."""
         chunk = model.get("chunk", None)
         if not chunk or chunk == -1:  # Avoid tedious log information
-            self.log.info(u"Saving file at %s", path)
+            self.log.info("Saving file at %s", path)
         model = await ensure_async(self.contents_manager.save(model, path))
         validate_model(model, expect_content=False)
         self._finish_model(model)
 
     @web.authenticated
+    @authorized
     async def post(self, path=""):
         """Create a new file in the specified path.
 
@@ -183,11 +206,17 @@ class ContentsHandler(APIHandler):
         if file_exists:
             raise web.HTTPError(400, "Cannot POST to files, use PUT instead.")
 
-        dir_exists = await ensure_async(cm.dir_exists(path))
-        if not dir_exists:
-            raise web.HTTPError(404, "No such directory: %s" % path)
-
         model = self.get_json_body()
+        copy_from = model.get("copy_from")
+        if (
+            copy_from
+            and (
+                await ensure_async(cm.is_hidden(path))
+                or await ensure_async(cm.is_hidden(copy_from))
+            )
+            and not cm.allow_hidden
+        ):
+            raise web.HTTPError(400, f"Cannot copy file or directory {path!r}")
 
         if model is not None:
             copy_from = model.get("copy_from")
@@ -201,6 +230,7 @@ class ContentsHandler(APIHandler):
             await self._new_untitled(path)
 
     @web.authenticated
+    @authorized
     async def put(self, path=""):
         """Saves the file in the location specified by name and path.
 
@@ -213,9 +243,17 @@ class ContentsHandler(APIHandler):
           create a new empty notebook.
         """
         model = self.get_json_body()
+        cm = self.contents_manager
+
         if model:
             if model.get("copy_from"):
                 raise web.HTTPError(400, "Cannot copy with PUT, only POST")
+            if (
+                (model.get("path") and await ensure_async(cm.is_hidden(model.get("path"))))
+                or await ensure_async(cm.is_hidden(path))
+            ) and not cm.allow_hidden:
+                raise web.HTTPError(400, f"Cannot create file or directory {path!r}")
+
             exists = await ensure_async(self.contents_manager.file_exists(path))
             if exists:
                 await self._save(model, path)
@@ -225,17 +263,23 @@ class ContentsHandler(APIHandler):
             await self._new_untitled(path)
 
     @web.authenticated
+    @authorized
     async def delete(self, path=""):
         """delete a file in the given path"""
         cm = self.contents_manager
+
+        if await ensure_async(cm.is_hidden(path)) and not cm.allow_hidden:
+            raise web.HTTPError(400, f"Cannot delete file or directory {path!r}")
+
         self.log.warning("delete %s", path)
         await ensure_async(cm.delete(path))
         self.set_status(204)
         self.finish()
 
 
-class CheckpointsHandler(APIHandler):
+class CheckpointsHandler(ContentsAPIHandler):
     @web.authenticated
+    @authorized
     async def get(self, path=""):
         """get lists checkpoints for a file"""
         cm = self.contents_manager
@@ -244,6 +288,7 @@ class CheckpointsHandler(APIHandler):
         self.finish(data)
 
     @web.authenticated
+    @authorized
     async def post(self, path=""):
         """post creates a new checkpoint"""
         cm = self.contents_manager
@@ -261,8 +306,9 @@ class CheckpointsHandler(APIHandler):
         self.finish(data)
 
 
-class ModifyCheckpointsHandler(APIHandler):
+class ModifyCheckpointsHandler(ContentsAPIHandler):
     @web.authenticated
+    @authorized
     async def post(self, path, checkpoint_id):
         """post restores a file from a checkpoint"""
         cm = self.contents_manager
@@ -271,6 +317,7 @@ class ModifyCheckpointsHandler(APIHandler):
         self.finish()
 
     @web.authenticated
+    @authorized
     async def delete(self, path, checkpoint_id):
         """delete clears a checkpoint for a given file"""
         cm = self.contents_manager
@@ -292,9 +339,10 @@ class NotebooksRedirectHandler(JupyterHandler):
 
 
 class TrustNotebooksHandler(JupyterHandler):
-    """ Handles trust/signing of notebooks """
+    """Handles trust/signing of notebooks"""
 
     @web.authenticated
+    @authorized(resource=AUTH_RESOURCE)
     async def post(self, path=""):
         cm = self.contents_manager
         await ensure_async(cm.trust_notebook(path))
@@ -309,10 +357,11 @@ class TrustNotebooksHandler(JupyterHandler):
 
 _checkpoint_id_regex = r"(?P<checkpoint_id>[\w-]+)"
 
+
 default_handlers = [
     (r"/api/contents%s/checkpoints" % path_regex, CheckpointsHandler),
     (
-        r"/api/contents%s/checkpoints/%s" % (path_regex, _checkpoint_id_regex),
+        rf"/api/contents{path_regex}/checkpoints/{_checkpoint_id_regex}",
         ModifyCheckpointsHandler,
     ),
     (r"/api/contents%s/trust" % path_regex, TrustNotebooksHandler),
