@@ -1,16 +1,12 @@
 """A mock extension named `mock2` for testing purposes.
 """
-
-
 # by the test functions.
+
+
 def _jupyter_server_extension_paths():
-    return [
-        {
-            'module': 'tests.extension.mockextensions.mock2'
-        }
-    ]
+    return [{"module": "tests.extension.mockextensions.mock2"}]
 
 
 def _load_jupyter_server_extension(serverapp):
     serverapp.mockII = True
-    serverapp.mock_shared = 'II'
+    serverapp.mock_shared = "II"
