@@ -161,8 +161,8 @@ def test_stop_extension(jp_serverapp, caplog):
     run_sync(jp_serverapp.cleanup_extensions())
     assert [msg for *_, msg in caplog.record_tuples] == [
         "Shutting down 1 extension",
-        f'{extension_name} | extension app "mockextension" stopping',
-        f'{extension_name} | extension app "mockextension" stopped',
+        f"{extension_name} | extension app 'mockextension' stopping",
+        f"{extension_name} | extension app 'mockextension' stopped",
     ]
 
     # check the shutdown method was called once
