@@ -1,6 +1,6 @@
 import pytest
-from jupyter_server.extension.utils import validate_extension
 
+from jupyter_server.extension.utils import validate_extension
 
 # Use ServerApps environment because it monkeypatches
 # jupyter_core.paths and provides a config directory
@@ -10,10 +10,10 @@ pytestmark = pytest.mark.usefixtures("jp_environ")
 
 def test_validate_extension():
     # enabled at sys level
-    assert validate_extension('tests.extension.mockextensions.mockext_sys')
+    assert validate_extension("tests.extension.mockextensions.mockext_sys")
     # enabled at sys, disabled at user
-    assert validate_extension('tests.extension.mockextensions.mockext_both')
+    assert validate_extension("tests.extension.mockextensions.mockext_both")
     # enabled at user
-    assert validate_extension('tests.extension.mockextensions.mockext_user')
+    assert validate_extension("tests.extension.mockextensions.mockext_user")
     # enabled at Python
-    assert validate_extension('tests.extension.mockextensions.mockext_py')
\ No newline at end of file
+    assert validate_extension("tests.extension.mockextensions.mockext_py")
