@@ -186,6 +186,7 @@ def init_gateway(monkeypatch):
     monkeypatch.setenv("JUPYTER_GATEWAY_HTTP_USER", mock_http_user)
     monkeypatch.setenv("JUPYTER_GATEWAY_REQUEST_TIMEOUT", "44.4")
     monkeypatch.setenv("JUPYTER_GATEWAY_CONNECT_TIMEOUT", "44.4")
+    monkeypatch.setenv("JUPYTER_GATEWAY_LAUNCH_TIMEOUT_PAD", "1.1")
     yield
     GatewayClient.clear_instance()
 
@@ -198,11 +199,10 @@ async def test_gateway_env_options(init_gateway, jp_serverapp):
         jp_serverapp.gateway_config.connect_timeout == jp_serverapp.gateway_config.request_timeout
     )
     assert jp_serverapp.gateway_config.connect_timeout == 44.4
+    assert jp_serverapp.gateway_config.launch_timeout_pad == 1.1
 
     GatewayClient.instance().init_static_args()
-    assert GatewayClient.instance().KERNEL_LAUNCH_TIMEOUT == int(
-        jp_serverapp.gateway_config.request_timeout
-    )
+    assert GatewayClient.instance().KERNEL_LAUNCH_TIMEOUT == 43
 
 
 async def test_gateway_cli_options(jp_configurable_serverapp):
@@ -211,6 +211,7 @@ async def test_gateway_cli_options(jp_configurable_serverapp):
         "--GatewayClient.http_user=" + mock_http_user,
         "--GatewayClient.connect_timeout=44.4",
         "--GatewayClient.request_timeout=96.0",
+        "--GatewayClient.launch_timeout_pad=5.1",
     ]
 
     GatewayClient.clear_instance()
@@ -221,10 +222,40 @@ async def test_gateway_cli_options(jp_configurable_serverapp):
     assert app.gateway_config.http_user == mock_http_user
     assert app.gateway_config.connect_timeout == 44.4
     assert app.gateway_config.request_timeout == 96.0
+    assert app.gateway_config.launch_timeout_pad == 5.1
     GatewayClient.instance().init_static_args()
     assert (
-        GatewayClient.instance().KERNEL_LAUNCH_TIMEOUT == 96
-    )  # Ensure KLT gets set from request-timeout
+        GatewayClient.instance().KERNEL_LAUNCH_TIMEOUT == 90
+    )  # Ensure KLT gets set from request-timeout - launch_timeout_pad
+    GatewayClient.clear_instance()
+
+
+@pytest.mark.parametrize(
+    "request_timeout,kernel_launch_timeout,expected_request_timeout,expected_kernel_launch_timeout",
+    [(50, 10, 50, 45), (10, 50, 55, 50)],
+)
+async def test_gateway_request_timeout_pad_option(
+    jp_configurable_serverapp,
+    monkeypatch,
+    request_timeout,
+    kernel_launch_timeout,
+    expected_request_timeout,
+    expected_kernel_launch_timeout,
+):
+    argv = [
+        f"--GatewayClient.request_timeout={request_timeout}",
+        "--GatewayClient.launch_timeout_pad=5",
+    ]
+
+    GatewayClient.clear_instance()
+    app = jp_configurable_serverapp(argv=argv)
+
+    monkeypatch.setattr(GatewayClient, "KERNEL_LAUNCH_TIMEOUT", kernel_launch_timeout)
+    GatewayClient.instance().init_static_args()
+
+    assert app.gateway_config.request_timeout == expected_request_timeout
+    assert GatewayClient.instance().KERNEL_LAUNCH_TIMEOUT == expected_kernel_launch_timeout
+
     GatewayClient.clear_instance()
 
 
